"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from .checkpointout import CheckpointOut, CheckpointOutTypedDict
from .eventout import EventOut, EventOutTypedDict
from .finetuneablemodel import FineTuneableModel
from .trainingparameters import TrainingParameters, TrainingParametersTypedDict
from .wandbintegrationout import WandbIntegrationOut, WandbIntegrationOutTypedDict
from enum import Enum
from mistral_dev.types import BaseModel
import pydantic
from typing import Final, List, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class DetailedJobOutStatus(str, Enum):
    r"""The current status of the fine-tuning job."""
    QUEUED = "QUEUED"
    STARTED = "STARTED"
    RUNNING = "RUNNING"
    FAILED = "FAILED"
    SUCCESS = "SUCCESS"
    CANCELLED = "CANCELLED"
    CANCELLATION_REQUESTED = "CANCELLATION_REQUESTED"


class DetailedJobOutTypedDict(TypedDict):
    id: str
    hyperparameters: TrainingParametersTypedDict
    r"""The fine-tuning hyperparameter settings used in a fine-tune job."""
    model: FineTuneableModel
    r"""The name of the model to fine-tune."""
    status: DetailedJobOutStatus
    r"""The current status of the fine-tuning job."""
    job_type: str
    r"""The type of job (`FT` for fine-tuning)."""
    created_at: int
    r"""The UNIX timestamp (in seconds) for when the fine-tuning job was created."""
    modified_at: int
    r"""The UNIX timestamp (in seconds) for when the fine-tuning job was last modified."""
    training_files: List[str]
    r"""A list containing the IDs of uploaded files that contain training data."""
    validation_files: NotRequired[List[str]]
    r"""A list containing the IDs of uploaded files that contain validation data."""
    fine_tuned_model: NotRequired[str]
    r"""The name of the fine-tuned model that is being created. The value will be `null` if the fine-tuning job is still running."""
    integrations: NotRequired[List[WandbIntegrationOutTypedDict]]
    r"""A list of integrations enabled for your fine-tuning job."""
    events: NotRequired[List[EventOutTypedDict]]
    r"""Event items are created every time the status of a fine-tuning job changes.
    The timestamped list of all events is accessible here.

    """
    checkpoints: NotRequired[List[CheckpointOutTypedDict]]
    

class DetailedJobOut(BaseModel):
    id: str
    hyperparameters: TrainingParameters
    r"""The fine-tuning hyperparameter settings used in a fine-tune job."""
    model: FineTuneableModel
    r"""The name of the model to fine-tune."""
    status: DetailedJobOutStatus
    r"""The current status of the fine-tuning job."""
    job_type: str
    r"""The type of job (`FT` for fine-tuning)."""
    created_at: int
    r"""The UNIX timestamp (in seconds) for when the fine-tuning job was created."""
    modified_at: int
    r"""The UNIX timestamp (in seconds) for when the fine-tuning job was last modified."""
    training_files: List[str]
    r"""A list containing the IDs of uploaded files that contain training data."""
    validation_files: Optional[List[str]] = None
    r"""A list containing the IDs of uploaded files that contain validation data."""
    OBJECT: Annotated[Final[Optional[str]], pydantic.Field(alias="object")] = "job"
    fine_tuned_model: Optional[str] = None
    r"""The name of the fine-tuned model that is being created. The value will be `null` if the fine-tuning job is still running."""
    integrations: Optional[List[WandbIntegrationOut]] = None
    r"""A list of integrations enabled for your fine-tuning job."""
    events: Optional[List[EventOut]] = None
    r"""Event items are created every time the status of a fine-tuning job changes.
    The timestamped list of all events is accessible here.

    """
    checkpoints: Optional[List[CheckpointOut]] = None
    
